@@ -43,6 +43,7 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <unistd.h>
 #include <locale.h>
 #include <string.h>
@@ -58,7 +59,7 @@ static gboolean        server_threaded = FALSE;
 static glong           server_guard_depth = 0;
 static GStaticRecMutex server_guard = G_STATIC_REC_MUTEX_INIT;
 
-PortableServer_POA
+static PortableServer_POA
 server_get_poa (void)
 {
         PortableServer_POA poa;
@@ -169,7 +170,10 @@ static const GOptionEntry options[] = {
 	 N_("File descriptor to write IOR to"), N_("FD")},
 
 	{"register-server", '\0', 0, G_OPTION_ARG_NONE, &server_reg,
-	 N_("Register as the users' activation server without locking [!] for debugging"),
+	 N_("Register as the user's activation server without locking."
+            "  Warning: this option can have dangerous side effects"
+            " on the stability of the user's running session,"
+            " and should only be used for debugging purposes"),
 	 NULL},
 
 #ifdef BONOBO_ACTIVATION_DEBUG
@@ -390,7 +394,9 @@ main (int argc, char *argv[])
 #ifdef HAVE_SYSLOG_H
 	gchar                        *syslog_ident;
 #endif
+#ifdef BONOBO_ACTIVATION_DEBUG
 	const gchar                  *debug_output_env;
+#endif
 	GError                       *error = NULL;
 
 #ifdef HAVE_SETSID
