@@ -37,12 +37,16 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <glib.h>
-#include <popt.h>
 #include <signal.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
+#ifdef G_OS_WIN32
+#include <io.h>
+#include <windows.h>
+#endif
+
 /****************** ORBit-specific stuff ****************/
 
 #include <orbit/orbit.h>
@@ -366,13 +370,37 @@ static const GOptionEntry bonobo_activation_goption_options[] = {
         { NULL }
 };
 
+static void
+init_gettext (gboolean bind_codeset)
+{
+	static gboolean initialized = FALSE;
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET	
+	static gboolean codeset_bound = FALSE;
+#endif
+
+	if (!initialized)
+	{
+		bindtextdomain (GETTEXT_PACKAGE, BONOBO_ACTIVATION_LOCALEDIR);
+		initialized = TRUE;
+	}
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET	
+	if (!codeset_bound && bind_codeset)
+	{
+		bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+		codeset_bound = TRUE;
+	}
+#endif
+}
+
 GOptionGroup *
 bonobo_activation_get_goption_group (void)
 {
 	GOptionGroup *group;
 
-	group = g_option_group_new ("bonbo-activation",
-				    N_("Bonobo Activation"),
+	init_gettext (TRUE);
+
+	group = g_option_group_new ("bonobo-activation",
+				    N_("Bonobo Activation options:"),
 				    N_("Show Bonobo Activation options"),
 				    NULL, NULL);
 	g_option_group_set_translation_domain (group, GETTEXT_PACKAGE);
@@ -405,7 +433,7 @@ bonobo_activation_ior_fd_get (void)
 void
 bonobo_activation_preinit (gpointer app, gpointer mod_info)
 {
-        bindtextdomain (GETTEXT_PACKAGE, BONOBO_ACTIVATION_LOCALEDIR);
+	init_gettext (FALSE);
 }
 
 void
@@ -417,9 +445,22 @@ bonobo_activation_postinit (gpointer app, gpointer mod_info)
 	if (bonobo_activation_ior_fd > 2)
 		fcntl (bonobo_activation_ior_fd, F_SETFD, FD_CLOEXEC);
 #endif
+#ifdef G_OS_WIN32
+	if (bonobo_activation_ior_fd > 2) {
+                HANDLE newhandle;
+
+                if (DuplicateHandle (GetCurrentProcess (), (HANDLE) _get_osfhandle (bonobo_activation_ior_fd),
+                                     GetCurrentProcess (), &newhandle,
+                                     0, FALSE, DUPLICATE_SAME_ACCESS)) {
+                        close (bonobo_activation_ior_fd);
+                        bonobo_activation_ior_fd = _open_osfhandle ((int) newhandle, _O_TEXT | _O_NOINHERIT);
+                }
+        }
+#endif
+
         if (bonobo_activation_activate_iid)
                 g_timeout_add_full (G_PRIORITY_LOW,
-                                    BONOBO_ACTIVATION_FACTORY_TIMEOUT,
+                                    BONOBO_ACTIVATION_FACTORY_PRIVATE_TIMEOUT,
                                     bonobo_activation_timeout_reg_check,
                                     NULL, NULL);
         else
@@ -467,8 +508,8 @@ bonobo_activation_is_initialized (void)
 char *
 bonobo_activation_get_popt_table_name (void)
 {
-        bindtextdomain (PACKAGE, BONOBO_ACTIVATION_LOCALEDIR);
-        return _("Bonobo activation options");
+	init_gettext (FALSE);
+	return _("Bonobo activation options");
 }
 
 
@@ -489,15 +530,19 @@ bonobo_activation_init (int argc, char **argv)
 	CORBA_ORB retval;
 	int i;
 
-	g_return_val_if_fail (is_initialized == FALSE, bonobo_activation_orb);
+        if (!is_initialized)
+        {
+                bindtextdomain (PACKAGE, BONOBO_ACTIVATION_LOCALEDIR);
 
-        bindtextdomain (PACKAGE, BONOBO_ACTIVATION_LOCALEDIR);
+                bonobo_activation_preinit (NULL, NULL);
 
-	bonobo_activation_preinit (NULL, NULL);
+                retval = bonobo_activation_orb_init (&argc, argv);
+        }
+	else
+		retval = bonobo_activation_orb;
 
-	retval = bonobo_activation_orb_init (&argc, argv);
-
-	/* Handle non-popt case */
+        /* Accumulate arguments over multiple inits. Sometimes we are
+         * initialized from GTK_MODULEs or gnome-vfs with bogus arguments */
 	for (i = 1; i < argc; i++) {
                 if (!strncmp ("--oaf-ior-fd=", argv[i],
                               strlen ("--oaf-ior-fd="))) {
@@ -516,7 +561,8 @@ bonobo_activation_init (int argc, char **argv)
                 }     
 	}
 
-	bonobo_activation_postinit (NULL, NULL);
+        if (!is_initialized)
+                bonobo_activation_postinit (NULL, NULL);
 
 	return retval;
 }
@@ -537,7 +583,7 @@ bonobo_activation_orb_init (int *argc, char **argv)
         CORBA_Context def_ctx;
 	CORBA_Environment ev;
 	const char *hostname;
-	const gchar *orb_id;
+	gchar *orb_id;
 
 	CORBA_exception_init (&ev);
 
@@ -593,7 +639,7 @@ bonobo_activation_orb_init (int *argc, char **argv)
  * bonobo_activation_debug_shutdown:
  * 
  *   A debugging function to shutdown the ORB and process
- * any reference count leaks that may have occured.
+ * any reference count leaks that may have occurred.
  * 
  * Return value: FALSE if there were leaks detected, else TRUE
  **/
